# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os

import pytest
from gusregon import GUS
from gusregon.gus import ENDPOINT, ENDPOINT_SANDBOX
from vcr import VCR


vcr = VCR(path_transformer=VCR.ensure_suffix('.yaml'),
          cassette_library_dir=os.path.join('tests', 'cassettes'))


@pytest.fixture
def client():
    return GUS(sandbox=True)


def test_required_api_key():
    with pytest.raises(AttributeError):
        GUS()


def test_different_endpoints():
    gus = GUS(sandbox=True)
    assert gus.service._binding_options.get('address') == ENDPOINT_SANDBOX

    gus = GUS(api_key='my_api_key')
    assert gus.service._binding_options.get('address') == ENDPOINT


@pytest.mark.parametrize('kwargs', [{'nip': '5170359458'}, {'regon': '180853177'}])
@vcr.use_cassette
def test_get_company_type_p_address(client, kwargs):
    data = client.get_address(**kwargs)
    assert 'ASSECO' in data['name']
    assert data['street_address'] == 'ul. Test-Krucza 14'
    assert data['postal_code'] == '35-322'
    assert data['city'] == 'Rzeszów'


@pytest.mark.parametrize('kwargs', [{'nip': '8991051697'}, {'regon': '022225213'}])
@vcr.use_cassette
def test_get_company_type_f_address(client, kwargs):
    data = client.get_address(**kwargs)
    assert 'MECHANIKA POJAZDOWA' in data['name']
    assert data['street_address'] == 'ul. Test-Krucza 24'
    assert data['postal_code'] == '50-502'
    assert data['city'] == 'Wrocław'


@vcr.use_cassette
def test_get_company_type_f_krupgn_address(client):
    data = client.get_address(nip='5881018622')
    assert 'PRZEDSIĘBIORSTWO BUDOWLANE' in data['name']
    assert data['street_address'] == 'ul. Test-Krucza 24 C'
    assert data['postal_code'] == '84-200'
    assert data['city'] == 'Wejherowo'


@pytest.mark.parametrize('kwargs', [{'nip': '5170359458'}, {'regon': '180853177'}])
@vcr.use_cassette
def test_get_company_type_p_details(client, kwargs):
    data = client.search(**kwargs)
    assert 'ASSECO' in data['nazwa']
    assert data['adsiedzulica_nazwa'] == 'ul. Test-Krucza'
    assert data['adsiedznumernieruchomosci'] == '14'
    assert data['adsiedzkodpocztowy'] == '35322'
    assert data['adsiedzmiejscowosc_nazwa'] == 'Rzeszów'
    assert data['podstawowaformaprawna_nazwa'] == 'OSOBA PRAWNA'


@pytest.mark.parametrize('kwargs', [{'nip': '8991051697'}, {'regon': '022225213'}])
@vcr.use_cassette
def test_get_company_type_f_details(client, kwargs):
    data = client.search(**kwargs)
    assert 'MECHANIKA POJAZDOWA' in data['nazwa']
    assert data['adsiedzulica_nazwa'] == 'ul. Test-Krucza'
    assert data['adsiedznumernieruchomosci'] == '24'
    assert data['adsiedzkodpocztowy'] == '50502'
    assert data['adsiedzmiejscowosc_nazwa'] == 'Wrocław'


@vcr.use_cassette
def test_get_company_type_f_other_details(client):
    data = client.search(regon='970442445')
    assert 'PRZY SĄDZIE REJONOWYM W ŻAGANIU' in data['nazwa']
    assert data['adsiedzulica_nazwa'] == 'ul. Test-Wilcza'
    assert data['adsiedznumernieruchomosci'] == '1'
    assert data['adsiedzkodpocztowy'] == '68100'
    assert data['adsiedzmiejscowosc_nazwa'] == 'Żagań'


@vcr.use_cassette
def test_get_company_type_lp_details(client):
    data = client.search(regon='01034470800689')
    assert 'AGENCJA NIERUCHOMOŚCI' in data['nazwa']
    assert data['adsiedzulica_nazwa'] == 'ul. Test-Wilcza'
    assert data['adsiedznumernieruchomosci'] == '7'
    assert data['adsiedzkodpocztowy'] == '35001'
    assert data['adsiedzmiejscowosc_nazwa'] == 'Rzeszów'


@pytest.mark.parametrize('kwargs', [{'nip': '5170359458'}, {'regon': '180853177'}])
@vcr.use_cassette
def test_get_pkd_type_p(client, kwargs):
    items = client.get_pkd(**kwargs)
    assert '6201Z' in {i['code'] for i in items}


@pytest.mark.parametrize('kwargs', [{'nip': '8991051697'}, {'regon': '022225213'}])
@vcr.use_cassette
def test_get_pkd_type_f(client, kwargs):
    items = client.get_pkd(**kwargs)
    codes = {i['code']: i['main'] for i in items}
    assert codes.get('4520Z') is True
    assert codes.get('4532Z') is False


@vcr.use_cassette
def test_get_pkd_type_lp(client):
    items = client.get_pkd(regon='01034470800689')
    codes = {i['code']: i['main'] for i in items}
    assert codes.get('6831Z') is True



from bs4 import BeautifulSoup, Tag
from requests import Session
from zeep import Client, Transport

from .version import __version__

WSDL = 'https://wyszukiwarkaregontest.stat.gov.pl/wsBIR/wsdl/UslugaBIRzewnPubl.xsd'
ENDPOINT = 'https://wyszukiwarkaregon.stat.gov.pl/wsBIR/UslugaBIRzewnPubl.svc'
ENDPOINT_SANDBOX = 'https://wyszukiwarkaregontest.stat.gov.pl/wsBIR/UslugaBIRzewnPubl.svc'


class GUS(object):
    endpoint = ENDPOINT
    headers = {'User-Agent': 'gusregon/%s' % __version__}
    pkd_report_type = {
        'F': 'PublDaneRaportDzialalnosciFizycznej',
        'P': 'PublDaneRaportDzialalnosciPrawnej'}
    report_type = {
        'F': {
            '1': 'PublDaneRaportDzialalnoscFizycznejCeidg',
            '2': 'PublDaneRaportDzialalnoscFizycznejRolnicza',
            '3': 'PublDaneRaportDzialalnoscFizycznejPozostala',
            '4': 'PublDaneRaportDzialalnoscFizycznejWKrupgn'},
        'LF': 'PublDaneRaportLokalnaFizycznej',
        'P': 'PublDaneRaportPrawna',
        'LP': 'PublDaneRaportLokalnaPrawnej'}

    def __init__(self, api_key=None, sandbox=False):
        if not any([api_key, sandbox]):
            raise AttributeError('Api key is required.')
        self.api_key = api_key
        self.sandbox = sandbox
        if sandbox:
            self.api_key = api_key or 'abcde12345abcde12345'
            self.endpoint = ENDPOINT_SANDBOX
        transport = Transport(session=Session())
        transport.session.headers = self.headers
        client = Client(WSDL, transport=transport)
        self.service = client.create_service('{http://tempuri.org/}e3', self.endpoint)
        self.headers.update({'sid': self._service('Zaloguj', self.api_key)})

    def _service(self, action, *args, **kwargs):
        service = getattr(self.service, action)
        return service(*args, **kwargs)

    def _remove_prefix(self, data):
        data = {item.name: item.get_text()
                for item in BeautifulSoup(data, 'lxml').dane if isinstance(item, Tag)}
        parsed_data = {}
        for name, value in data.items():
            parsed_data[name.split('_', 1)[1]] = value.strip()
        return parsed_data

    def _get_details(self, nip=None, regon=None, krs=None):
        if not any([nip, regon, krs]):
            raise AttributeError(
                'At least one parameter (nip, regon, krs) is required.')
        if nip:
            search_params = {'Nip': nip}
        elif regon:
            search_params = {'Regon': regon}
        else:
            search_params = {'Krs': krs}
        return self._service('DaneSzukaj', search_params)

    def search(self, *args, **kwargs):
        details = self._get_details(*args, **kwargs)
        if details is not None:
            data = BeautifulSoup(details, 'lxml')
            report_type = self.report_type.get(data.typ.get_text())
            if isinstance(report_type, dict):
                report_type = report_type.get(data.silosid.get_text())
            return self._remove_prefix(self._service(
                'DanePobierzPelnyRaport', data.regon.get_text(), report_type))

    def get_pkd(self, *args, **kwargs):
        pkd = []
        details = self._get_details(*args, **kwargs)
        if details is not None:
            data = BeautifulSoup(details, 'lxml')
            report_type = self.pkd_report_type.get('F')
            if 'P' in data.typ.get_text():
                report_type = self.pkd_report_type.get('P')
            report = self._service(
                'DanePobierzPelnyRaport', data.regon.get_text(), report_type)
            if report is not None:
                for item in BeautifulSoup(report, 'lxml').find_all('dane'):
                    data = {i.name.split('_', 1)[1].replace('_', '').lower(): i.get_text()
                            for i in item.children if isinstance(i, Tag)}
                    pkd.append({
                        'code': data['pkdkod'],
                        'name': data['pkdnazwa'],
                        'main': data['pkdprzewazajace'] == '1'})
                pkd = [dict(t) for t in set([tuple(d.items()) for d in pkd])]
        return pkd

    def get_address(self, *args, **kwargs):
        details = self.search(*args, **kwargs)
        if details:
            postal_code = details['adsiedzkodpocztowy']
            address = '%s %s' % (details['adsiedzulica_nazwa'],
                                 details['adsiedznumernieruchomosci'])
            if details['adsiedznumerlokalu']:
                address += '/%s' % details['adsiedznumerlokalu']
            return {
                'name': details['nazwa'],
                'street_address': address,
                'postal_code': '%s-%s' % (postal_code[:2], postal_code[2:]),
                'city': details['adsiedzmiejscowosc_nazwa']}



# -*- coding: utf-8 -*-
import sys

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

exec(open("./gusregon/version.py").read())


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]
    test_args = []

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


setup(
    name='gusregon',
    version=__version__,
    description='GUS REGON Internet Database Client',
    long_description=open('README.rst').read(),
    author='Adam Bogdał',
    author_email='adam@bogdal.pl',
    url='https://github.com/bogdal/gusregon',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities'],
    install_requires=[
        'beautifulsoup4>=4.5.1',
        'lxml>=3.6.4',
        'zeep>=1.6.0'
    ],
    cmdclass={
        'test': PyTest
    },
    tests_require=[
        'pytest>=2.8.1',
        'pytest-cov',
        'vcrpy>=1.7.3'
    ]
)
